var FdLegacy = new function() {
    this.migrate = function() {
        var file = FdFile.getDataDirectory();
        file.append("fastdial.sqlite");
        if (file.exists()) {
            convertSqlite(file);
            file.moveTo(file.parent, file.leafName + ".bak");
        }
        var previousVersion = FdPrefs.getString("version");
        var version = "2.23b3";
        FdPrefs.setString("version", version);
        if (isOlder(previousVersion, "2.15")) {
            deleteImages();
            migrateOptions();
        }
        if (isOlder(previousVersion, "2.23")) {
            var options = FdPrefs.getObject("options");
            if (options.hideSearch != true) {
                var engine = Fd.getSearchEngine();
                var search = [ engine.name ];
                FdPrefs.setObject("search", search);
                delete options.hideSearch;
            }
        }
        if (!previousVersion) {
            addToolbarButton();
            FdUtils.openLink(FdInfo.URI, "tab");
        }
        if (version != previousVersion) {
            initThemes();
            FdUtils.openLink("about:fastdial", "tab");
        }
    }
    function isOlder(version1, version2) {
        if (!version1) return false;
        var parts1 = version1.split(/[^\d]+/);
        var parts2 = version2.split(/[^\d]+/);
        for (var i = 0; i < parts1.length; i++) {
            var v1 = parseInt(parts1[i]);
            var v2 = parseInt(parts2[i]);
            if (v1 != v2)
                return v1 < v2;
        }
    }

    function addToolbarButton() {
        var navbar = document.getElementById("nav-bar");
        var currentSet = navbar.currentSet;
        if (!currentSet.match("fd-button")) {
            currentSet = currentSet.replace(/urlbar-container/,
                    "fd-button,urlbar-container");
            navbar.setAttribute("currentset", currentSet);
            document.persist("nav-bar", "currentset");
        }
    }

    function deleteImages() {
        var dir = FdFile.getDataDirectory();
        FdFile.forEachFile(dir, function(file) {
            if (!file.leafName.match(/\.png$/)) return;
            try {
                file.remove(false);
            }
            catch(e) {
            }
        });
    }

    function initThemes() {
        var dir = FdFile.getExtensionDirectory();
        dir.append("chrome");
        dir.append("skin");
        dir.append("themes");
        FdFile.forEachFile(dir, FdTheme.import);
    }

    function migrateOptions() {
        var root = FdStorage.getRoot();
        var style = root.style || FdPrefs.getString("style");
        if (style) {
            FdTheme.remove("current");
            FdTheme.setStyle(convertStyle(style));
            FdPrefs.setString("theme", "~");
            FdTheme.copy("current", "~");
        }
        var options = FdPrefs.getObject("options");
        options.hideEmpty = FdPrefs.getBool("showEmpty") == false;
        var names = [ "width", "height", "thumbWidth", "fixed", "openIn", "shortcutKey" ];
        for (var i in names) {
            var name = names[i];
            if (root[name]) {
                options[name] = root[name];
            }
        }
        FdPrefs.setObject("options", options);
        var ids = FdBookmark.getAnnotatedIds(FdInfo.ID);
        for (var i in ids) {
            var item = FdBookmark.getBookmark(ids[i]);
            var annotation = FdBookmark.getAnnotation(item.id, FdInfo.ID);
            FdUtils.merge(item, FdUtils.fromJSON(annotation));
            convertItem(item);
            FdStorage.saveItem(item);
        }
        var enable = FdPrefs.getBool("showInNewTabs") != false;
        FdPrefs.setBool("enable", enable);
        FdPrefs.clear("showInNewTabs");
        FdPrefs.clear("showEmpty");
        FdPrefs.clear("showRefreshAll");
        FdPrefs.clear("useJava");
        FdPrefs.clear("refreshed");
        FdPrefs.clear("style");
    }

    function convertStyle(style) {
        if (/.title {.*?color: transparent;.*}/.test(style)) {
            style += " .title img { display: none; }";
        }
        if (/.hover .title {.*?color: transparent;.*}/.test(style)) {
            style += " div.thumbnail:hover .title img, .hover .title img { display: none; }";
        }
        return style;
    }

    function convertStyle2(style) {
        if (/.title {.*?color: transparent;.*}/.test(style)) {
            style += " .title img { display: none; }";
        }
        if (/.hover .title {.*?color: transparent;.*}/.test(style)) {
            style += " div.thumbnail:hover .title img, .hover .title img { display: none; }";
        }
        return style;
    }

    function convertItem(item) {
        item.thumbIndex = item.index;
        var names = [ "width", "height", "thumbWidth", "fixed", "openIn",
            "customImage", "customBackground", "style", "slow", "refreshed" ];
        for (var i in names) {
            var name = names[i];
            if (item[name] && name == "customImage") {
                item.logo = item[name];
            }
            delete item[name];
        }
    }

    function getHomePage() {
        return FdPrefs.getGlobalString("browser.startup.homepage");
    }

    function setHomePage(url) {
        FdPrefs.setGlobalString("browser.startup.homepage", url);
    }

    function convertSqlite(dbFile) {
        FdPrefs.setString("style", getStyle());
        var items = getItems(dbFile);
        var root = FdBookmark.createRoot();
        var translatedIds = [ root.id ];

        function saveItem(id) {
            if (translatedIds[id]) return; // Folder already migrated
            var item = items[id];
            if (!item) return;
            if (id == 0) {
                delete item["index"];
                delete item["title"];
                FdUtils.merge(root, item);
                FdStorage.saveItem(root);
                return;
            }
            if (!translatedIds[item.folderId]) saveItem(item.folderId);
            item.folderId = translatedIds[item.folderId];
            if (!item.folderId) return;
            FdStorage.saveItem(item);
            if (item.isFolder) translatedIds[id] = item.id;
            var file = FdFile.getDataDirectory();
            file.append(id + ".png");
            try {
                file.moveTo(file.parent, item.id + ".legacy");
            }
            catch(e) {
            }
            file.leafName = id + "_big.png";
            try {
                file.moveTo(file.parent, item.id + "-zoom.legacy");
            }
            catch(e) {
            }
        }

        for (var i in items) saveItem(i);
        var dir = FdFile.getDataDirectory();
        FdFile.forEachFile(dir, function(file) {
            var match = file.leafName.match(/^(.*)\.legacy$/);
            if (match) {
                file.moveTo(file.parent, match[1] + ".png");
            }
        });
    }

    function getItems(dbFile) {
        var storageService = Components.classes["@mozilla.org/storage/service;1"]
                .getService(Components.interfaces.mozIStorageService);
        var connection = storageService.openDatabase(dbFile);
        var text = "text: ";

        function getString(statement, id) {
            var value = statement.getString(id);
            return value && value.substr(0, text.length) == text
                    ? value.substr(text.length) : value;
        }

        var sql = "SELECT id, parentId, indexx, url, title, refresh, customImage," +
                "customBackground, isParent, width, height, thumbWidth " +
                "FROM pageInfos ORDER BY parentId";
        var statement = connection.createStatement(sql);
        var items = {};
        while (statement.executeStep()) {
            var item = {
                folderId:         statement.getInt32(1),
                index:            statement.getInt32(2) - 1,
                url:              getString(statement, 3),
                title:            getString(statement, 4),
                refresh:          statement.getInt32(5) && FdPrefs.getInt("refresh"),
                customImage:      getString(statement, 6),
                customBackground: getString(statement, 7),
                isFolder:         statement.getInt32(8)
            }
            if (item.isFolder) {
                item.width = statement.getInt32(9);
                item.height = statement.getInt32(10);
                item.thumbWidth = statement.getInt32(11);
                item.fixed = FdPrefs.getBool("fixedSize");
            }
            items[statement.getInt32(0)] = item;
        }
        statement.finalize();
        connection.close();
        return items;
    }

    function getStyle() {
        var theme = FdPrefs.getString("theme");
        if (!theme) return "";
        var old = FdUtils.fromJSON(theme).style;
        var style = {};
        style["body"] = {
            "background-image"  : old.body.backgroundImage,
            "background-repeat" : "repeat",
            "font-family"       : old.body.fontFamily ? "'" + old.body.fontFamily + "'" : "",
            "font-size"         : old.body.fontSize,
            "background-color"  : old.body.backgroundColor,
            "color"             : old.body.color
        }
        style[".thumbnail"] = {
            "background-color"   : old.image.backgroundColor,
            "border"             : old.cell.border,
        }
        style[".title"] = {
            "background-color" : old.cell.backgroundColor,
            "border-top"       : old.cell.border
        }
        style["div.thumbnail:hover, .hover"] = {
            "background-color" : old.image.hover.backgroundColor,
            "border"           : old.cell.hover.border,
        }
        style["div.thumbnail:hover .title, .hover .title"] = {
            "background-color" : old.cell.hover.backgroundColor,
            "border-top"       : old.cell.hover.border
        }
        var data = FdURL.readURL("chrome://fastdial/content/template/style.tpl");
        var template = new JsTemplate.Template(data);
        return template.run({ style: style });
    }
}
