var FdStorage = new function() {
    this.SHORTCUT_KEY = "fastdial/shortcutKey";
    this.REFRESH = "fastdial/refresh";
    this.getFolderURL = function(id) {
        if (id) return FdInfo.URI + "?folder=" + id;
        return null;
    };
    this.getURL = function(item) {
        return item.id && item.isFolder
                ? FdStorage.getFolderURL(item.id) : item.url;
    };
    this.getRoot = function() {
        var root = FdBookmark.getRoot();
        getProperties(root);
        return root;
    };
    function getProperties(bookmark) {
        var annotation = FdBookmark.getAnnotation(bookmark.id, FdInfo.ID);
        FdUtils.merge(bookmark, FdUtils.fromJSON(annotation));
        bookmark.shortcutKey = FdBookmark.getAnnotation(bookmark.id, FdStorage.SHORTCUT_KEY);
        bookmark.refresh = FdBookmark.getAnnotation(bookmark.id, FdStorage.REFRESH);
    }

    function saveProperties(item) {
        if (item.id == -1) return;
        var annotation = FdUtils.clone(item);
        var exclude = ["id", "folderId", "isFolder", "url", "title",
            "index", "feed", "description", "shortcutKey", "refresh"];
        for (var i in exclude) {
            delete annotation[exclude[i]];
        }
        FdBookmark.setAnnotation(item.id, FdInfo.ID, FdUtils.toJSON(annotation));
        FdBookmark.setAnnotation(item.id, FdStorage.SHORTCUT_KEY, item.shortcutKey);
        FdBookmark.setAnnotation(item.id, FdStorage.REFRESH, item.refresh);
    }

    this.getItem = function(id) {
        var bookmark = FdBookmark.getBookmark(id);
        getProperties(bookmark);
        return bookmark;
    };
    this.getItems = function(folderId, order, maxResults) {
        var sort = order == "title" ? FdBookmark.SORT_TITLE :
                order == "visits" ? FdBookmark.SORT_VISITS : 0;
        var items = [];
        var bookmarks = FdBookmark.getBookmarks(folderId, sort, maxResults);
        for (var i = 0; i < bookmarks.length; i++) {
            var bookmark = bookmarks[i];
            getProperties(bookmark);
            if (order) items.push(bookmark);
            else {
                if (bookmark.thumbIndex == undefined || items[bookmark.thumbIndex]) {
                    bookmark.thumbIndex = getFreeIndex(items);
                    if (bookmark.folderId != -1) saveProperties(bookmark);
                }
                items[bookmark.thumbIndex] = bookmark;
            }
        }
        return items;
    };
    function getFreeIndex(items) {
        for (var i = 0; ; i++) {
            if (!items[i]) return i;
        }
    }

    this.saveItem = function(item) {
        FdBookmark.saveBookmark(item);
        saveProperties(item);
    };
    this.removeItem = function(id) {
        FdBookmark.removeAnnotation(id, this.SHORTCUT_KEY);
        FdBookmark.removeAnnotation(id, this.REFRESH);
        FdBookmark.removeAnnotation(id, FdInfo.ID);
        FdBookmark.removeBookmark(id);
    };
    this.getAnnotatedItems = function(annotation) {
        var items = [];
        var ids = FdBookmark.getAnnotatedIds(annotation);
        for (var i in ids) {
            var item = this.getItem(ids[i]);
            items.push(item);
        }
        return items;
    };
    this.export = function(folderId) {
        var file = FdFile.chooseFile("save", ["*.json"]);
        if (!file) return;
        if (!file.leafName.match(/\.json$/)) {
            file.leafName += ".json";
        }
        var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
        stream.init(file, 0x02 | 0x08 | 0x20, 0x180, 0);
        var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
                createInstance(Components.interfaces.nsIConverterOutputStream);
        converter.init(stream, "UTF-8", 0, 0x0000);
        var streamProxy = {
            converter: converter,
            write: function(aData, aLen) {
                this.converter.writeString(aData);
            }
        };
        var result = FdBookmark.query(folderId);
        result.root.containerOpen = true;
        PlacesUtils.serializeNodeAsJSONToOutputStream(result.root, streamProxy, false, false);
        result.root.containerOpen = false;
        converter.close();
        stream.close();
    }
    this.import = function(folderId) {
        var file = FdFile.chooseFile("open", ["*.json"]);
        if (!file) return;
        var data = FdFile.readFile(file);
        var nodes = PlacesUtils.unwrapNodes(data, PlacesUtils.TYPE_X_MOZ_PLACE_CONTAINER);
        PlacesUtils.importJSONNode(nodes[0], folderId, -1);
    }
}
